Package: nleqslv
Version: 3.3.7
Title: Solve Systems of Nonlinear Equations
Authors@R: 
    c(person(given = "Berend",
           family = "Hasselman",
           role = c("aut"),
           email = "bhh@xs4all.nl",
           comment = "Original Author"),
      person(given = "Rob",
             family = "Carnell",
             role = c("aut", "cre"),
             email = "bertcarnell@gmail.com",
             comment = c(ORCID = "0009-0009-0465-7564")))
Description: Solve a system of nonlinear equations using a Broyden or a Newton method
             with a choice of global strategies such as line search and trust region.
             There are options for using a numerical or user supplied Jacobian,
             for specifying a banded numerical Jacobian and for allowing
             a singular or ill-conditioned Jacobian.
License: GPL (>= 2)
Encoding: UTF-8
URL: https://bertcarnell.github.io/nleqslv/,
        https://github.com/bertcarnell/nleqslv/
BugReports: https://github.com/bertcarnell/nleqslv/issues/
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-10 01:02:32 UTC; bertc
Author: Berend Hasselman [aut] (Original Author),
  Rob Carnell [aut, cre] (ORCID: <https://orcid.org/0009-0009-0465-7564>)
Maintainer: Rob Carnell <bertcarnell@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-10 05:20:02 UTC
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
